@@ -28,6 +28,7 @@ import os
 import warnings
 import sys
 import weakref
+import collections
 
 import ctypes
 from ctypes import c_char_p, POINTER, c_size_t, c_bool, c_void_p, c_int
@@ -82,8 +83,8 @@ class MatlabSession(object):
         Access to the MATLAB output buffer.
     workspace : Workspace object
         Easy access to MATLAB workspace, e.g. `workspace.sin([1.,2.,3.])`.
-    version : str
-        MATLAB version number.
+    version : tuple or None
+        MATLAB/libeng version number.
 
     Methods
     -------
@@ -108,7 +109,7 @@ class MatlabSession(object):
         self._matlab_root = matlab_root
 
 
-        engine, libeng, libmx = load_engine_and_libs(matlab_root, options)
+        engine, libeng, libmx, version = load_engine_and_libs(matlab_root, options)
 
         self._libeng = libeng
         self._libmx = libmx
@@ -116,6 +117,11 @@ class MatlabSession(object):
 
 
 
+        ### MATLAB/libeng version
+        self.version = version
+
+
+
         ### Setup the output buffer
         if buffer_size != 0:
             self._output_buffer = ctypes.create_string_buffer(buffer_size)
@@ -134,11 +140,12 @@ class MatlabSession(object):
 
 
 
-
     def __del__(self):
         self._libeng.engClose(self._ep)
 
 
+
+
     @property
     def output_buffer(self):
         if self._output_buffer is None:
@@ -147,6 +154,7 @@ class MatlabSession(object):
             return self._output_buffer.value
 
 
+
     def eval(self, expression):
         """Evaluate `expression` in MATLAB engine.
 
@@ -211,15 +219,6 @@ class MatlabSession(object):
         self._libmx.mxDestroyArray(pm)
 
 
-    @property
-    def version(self):
-        """Return string representing MATLAB version."""
-
-        self.eval("VERSION__ = version")
-        ver = self.get('VERSION__')
-        self.eval("clear VERSION__")
-
-        return ver
 
 
 
@@ -278,7 +277,7 @@ def load_engine_and_libs(matlab_root, options):
         if bits == '64bit':
             lib_dir = join(matlab_root, "bin", "glnxa64")
         else:
-            unsopported_paltform()
+            unsupported_platform(system, bits)
 
         libeng = Library(
             join(lib_dir, 'libeng.so')
@@ -292,12 +291,16 @@ def load_engine_and_libs(matlab_root, options):
             options=options
         )
 
+        ### Check for /bin/csh
+        if not os.path.exists("/bin/csh"):
+            warnings.warn("MATLAB engine requires /bin/csh.  Please, install it on your system or matlab_wrapper will not work properly.")
+
 
     elif system == 'Windows':
         if bits == '64bit':
             lib_dir = join(matlab_root, "bin", "win64")
         else:
-            unsopported_paltform()
+            unsupported_platform(system, bits)
 
         if lib_dir not in os.environ['PATH']:
             os.environ['PATH'] = lib_dir + ';' + os.environ['PATH']
@@ -312,7 +315,7 @@ def load_engine_and_libs(matlab_root, options):
         if bits == '64bit':
             lib_dir = join(matlab_root, "bin", "maci64")
         else:
-            unsopported_paltform(system,bits)
+            unsupported_platform(system, bits)
 
         libeng = Library(
             join(lib_dir, 'libeng.dylib')
@@ -328,7 +331,7 @@ def load_engine_and_libs(matlab_root, options):
 
 
     else:
-        unsopported_paltform()
+        unsupported_platform(system, bits)
 
 
 
@@ -338,30 +341,34 @@ def load_engine_and_libs(matlab_root, options):
         version_str = c_char_p.in_dll(libeng, "libeng_version").value
         version = tuple([int(v) for v in version_str.split('.')[:2]])
 
-
-        if (system == 'Linux') and (version == (8,1)) and (bits == '64bit'):
-            pass
-
-        elif (system == 'Linux') and (version == (8,2)) and (bits == '64bit'):
-            pass
-
-        elif (system == 'Linux') and (version == (8,3)) and (bits == '64bit'):
-            warnings.warn("You are using MATLAB version 8.3 (R2014a) on Linux, which appears to have a bug in engGetVariable().  You will probably only be able to use arrays of type double.")
-
-        elif (system == 'Windows') and (version == (8,3)) and (bits == '64bit'):
-            pass
-
-        elif (system == 'Darwin') and (version == (8,1)) and (bits == '64bit'):
-            pass
-
-        elif (system == 'Darwin') and (version == (8,3)) and (bits == '64bit'):
-            warnings.warn("You are using MATLAB version 8.3 (R2014a) on OS X, which appears to have a bug in engGetVariable().  You will probably only be able to use arrays of type double.")
-
-        else:
-            warnings.warn("Hi! You are using MATLAB version that was never tested with matlab_wrapper.  Please, let us know, that are using MATLAB version {version} on {os} with {bits}: https://github.com/mrkrd/matlab_wrapper".format(version=version_str,os=system,bits=bits))
-
     except ValueError:
-        warnings.warn("Unknown MATLAB version, please let us know: https://github.com/mrkrd/matlab_wrapper")
+        warnings.warn("Unable to identify MATLAB version, please let us know: https://github.com/mrkrd/matlab_wrapper")
+        version = None
+
+
+    if (system == 'Linux') and (version == (8,1)) and (bits == '64bit'):
+        pass
+
+    elif (system == 'Linux') and (version == (8,2)) and (bits == '64bit'):
+        pass
+
+    elif (system == 'Linux') and (version == (8,3)) and (bits == '64bit'):
+        warnings.warn("You are using MATLAB version 8.3 (R2014a) on Linux, which appears to have a bug in engGetVariable().  You will only be able to use arrays of type double.")
+
+    elif (system == 'Linux') and (version == (8,4)) and (bits == '64bit'):
+        pass
+
+    elif (system == 'Windows') and (version == (8,3)) and (bits == '64bit'):
+        pass
+
+    elif (system == 'Darwin') and (version == (8,1)) and (bits == '64bit'):
+        pass
+
+    elif (system == 'Darwin') and (version == (8,3)) and (bits == '64bit'):
+        warnings.warn("You are using MATLAB version 8.3 (R2014a) on OS X, which appears to have a bug in engGetVariable().  You will only be able to use arrays of type double.")
+
+    else:
+        warnings.warn("Hi! You are using MATLAB version that was never tested with matlab_wrapper.  Please, let us know about that and visit out website <https://github.com/mrkrd/matlab_wrapper> or send me an email <marekrud@gmail.com>.  Your MATLAB version is {version} on {os} with {bits}.".format(version=version_str,os=system,bits=bits))
 
 
 
@@ -369,12 +376,12 @@ def load_engine_and_libs(matlab_root, options):
     engine = libeng.engOpen(command)
 
 
-    return engine, libeng, libmx
+    return engine, libeng, libmx, version
 
 
 
 
-def unsopported_paltform(system, bits):
+def unsupported_platform(system, bits):
     raise RuntimeError("""Unsopported OS or architecture: {} {}.
 
 Please, check our website about supported platforms:
@@ -682,8 +689,10 @@ def mxarray_to_ndarray(libmx, pm):
 
 def ndarray_to_mxarray(libmx, arr):
 
+    ### Prepare `arr` object (convert to ndarray if possible), assert
+    ### data type
     if isinstance(arr, str):
-        pm = libmx.mxCreateString(arr)
+        pass
 
     elif isinstance(arr, dict):
         raise NotImplementedError('dicts are not supported.')
@@ -694,11 +703,23 @@ def ndarray_to_mxarray(libmx, arr):
     elif ('pandas' in sys.modules) and isinstance(arr, sys.modules['pandas'].Series):
         arr = arr.to_frame().to_records()
 
-    else:
+    elif isinstance(arr, collections.Iterable):
         arr = np.array(arr, ndmin=2)
 
+    elif np.issctype(type(arr)):
+        arr = np.array(arr, ndmin=2)
 
-    if isinstance(arr, np.ndarray) and arr.dtype.kind in ['i','u','f','c']:
+    else:
+        raise NotImplementedError("Data type not supported: {}".format(type(arr)))
+
+
+
+
+    ### Convert ndarray to mxarray
+    if isinstance(arr, str):
+        pm = libmx.mxCreateString(arr)
+
+    elif isinstance(arr, np.ndarray) and arr.dtype.kind in ['i','u','f','c']:
         dim = arr.ctypes.shape_as(mwSize)
         complex_flag = (arr.dtype.kind == 'c')
 
@@ -761,7 +782,7 @@ def ndarray_to_mxarray(libmx, arr):
                 libmx.mxSetField(pm, i, name, p)
 
     elif isinstance(arr, np.ndarray):
-        raise NotImplementedError('Type {} not supported.'.format(arr.dtype))
+        raise NotImplementedError('Unsupported dtype: {}'.format(arr.dtype))
 
     return pm
 
@@ -801,7 +822,11 @@ class Library(object):
 
             self.engOutputBuffer.argtypes = (POINTER(Engine), c_char_p, c_int)
             self.engOutputBuffer.restype = c_int
+            self.engOutputBuffer.errcheck = error_check
 
+            self.engClose.argtypes = (POINTER(Engine),)
+            self.engClose.restype = c_int
+            self.engClose.errcheck = error_check
 
         elif 'libmx' in name:
 
